class CallNotFound(Exception):
    pass
class InvalidAudioURL(Exception):
    pass
class InvalidToken(Exception):
    pass
class EndpoitNotFound(Exception):
    pass
class ServerSideError(Exception):
    pass
class InvalidEndpoint(Exception):
    pass
class CallNotActive(Exception):
    pass
class NoMoreCallsAllowed(Exception):
    pass
class OutboundCallReachesLimit(Exception):
    pass
class InvalidPhoneNumber(Exception):
    pass
class TokenRequired(Exception):
    pass
class InvalidCallUUID(Exception):
    pass
class InvalidAudioFile(Exception):
    pass
class InvalidParameter(Exception):
    pass
class InvalidState(Exception):
    pass
class MissingCallingParameters(Exception):
    pass
class CountryNotAvailable(Exception):
    pass
